/*
**	keyWord.c
*/
#include <string.h>
#include "opencasl.h"

/*
**	opeCode, opeType, opeSize, mnemonic
**
**	opeType 0	GR, adr[, XR] + GR, GR
**			1	GR, adr[, XR]
**			2	adr[, XR]
**			3	GR
**			4	adr
**			5	[adr]
**			6	None
**			7	DS
**			8	DC
**			9	IN, OUT
*/
static	MNEMONIC	mnemo[] = {
	{CMD_NOP,	6, 2, "NOP"},
	{CMD_LD,	0, 2, "LD"},
	{CMD_ST,	1, 2, "ST"},
	{CMD_LAD,	1, 2, "LAD"},
	{CMD_ADDA,	0, 2, "ADDA"},
	{CMD_ADDL,	0, 2, "ADDL"},
	{CMD_SUBA,	0, 2, "SUBA"},
	{CMD_SUBL,	0, 2, "SUBL"},
	{CMD_AND,	0, 2, "AND"},
	{CMD_OR,	0, 2, "OR"},
	{CMD_XOR,	0, 2, "XOR"},
	{CMD_CPA,	0, 2, "CPA"},
	{CMD_CPL,	0, 2, "CPL"},
	{CMD_SLA,	0, 2, "SLA"},
	{CMD_SRA,	0, 2, "SRA"},
	{CMD_SLL,	0, 2, "SLL"},
	{CMD_SRL,	0, 2, "SRL"},
	{CMD_JPL,	2, 2, "JPL"},
	{CMD_JMI,	2, 2, "JMI"},
	{CMD_JNZ,	2, 2, "JNZ"},
	{CMD_JZE,	2, 2, "JZE"},
	{CMD_JOV,	2, 2, "JOV"},
	{CMD_JUMP,	2, 2, "JUMP"},
	{CMD_PUSH,	2, 2, "PUSH"},
	{CMD_POP,	3, 2, "POP"},
	{CMD_CALL,	2, 2, "CALL"},
	{CMD_RET,	6, 2, "RET"},
	{CMD_INPUT,	4, 2, "INPUT"},
	{CMD_OUTPUT,4, 2, "OUTPUT"},
	{CMD_RPUSH, 6, 2, "RPUSH"},
	{CMD_RPOP,  6, 2, "RPOP"},
	{CMD_SVC,	2, 2, "SVC"},
	{CMD_IN,	9, 2, "IN"},
	{CMD_OUT,	9, 2, "OUT"},
	{CMD_EXIT,	6, 2, "EXIT"},
	{CMD_START, 5, 0, "START"},
	{CMD_END,	6, 0, "END"},
	{CMD_DS,	7, 1, "DS"},
	{CMD_DC,	8, 1, "DC"}
};

static int	size_mnemonic = sizeof(mnemo)/sizeof(MNEMONIC);

int isMnemonic(char *name)
{
	int		i;
	int		result = ERROR;

	for (i = 0; i < size_mnemonic; i++) {
		if (strcmp(name, mnemo[i].key) == 0) {
			result = i;
			break;
		}
	}
	return result;
}

word getOpeCode(int pos)
{
	return mnemo[pos].opeCode;
}

int getOpeType(int pos)
{
	return mnemo[pos].opeType;
}

int getOpeSize(int pos)
{
	return mnemo[pos].opeSize;
}
