/*
**	oca.c
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "opencasl.h"

/*	execute assemble	*/
char	inbuf[BUFSIZ];
FILE	*obj, *prn, *sym;
int		lineNo;
char	errorMsg[BUFSIZ];
int		errorFlag, nError;
word	lastAddress;

int main(int argc, char *argv[])
{
	int		i;
	int		result = 0;
	char	*p;

	/*	init	*/
	initLable();

	/*	main loop */

	for (i = 1; i < argc; i++) {
		p = argv[i];
		if (*p == '-') {
			p++;
			switch(*p) {
			case 'h':
				dispHelp();
				exit(0);
			default:
				break;
			}
		} else {
			assemble(argv[i]);
		}
	}
	/*	final	*/
	deleteLabel();
	
	return result;
}

void assemble(char* fname)
{
	if (pass1(fname) == NOERROR) {
		pass2(fname);
	}
}

/*	make symbol table	*/
int pass1(char *fname)
{
	int		p, n, pos;
	FILE	*fp;
	char	buf[BUFSIZ];
	char	*items[MAXITEMS];
	word	address = ORGADDR;

	if ((fp = fopen(fname, "r")) == NULL) {
		return ERROR;
	}
	
	clearLabels();
	while (fgets(buf, BUFSIZ, fp)) {
		n = dividString(items, buf);
		if (0 < n) {
			pos = isMnemonic(items[p = 0]);
			if (pos < 0) {
				addLabel(items[p++], address);
				if (p < n) {
					pos = isMnemonic(items[p]);
				}
			}
			if (0 <= pos) {
				if (getOpeCode(pos) == CMD_DS && p + 1 < n) {
					address += getWord(items[p + 1]);
				} else if (getOpeCode(pos) == CMD_DC && p + 1 < n) {
					address += getDCSize(items, p + 1, n);
				} else if (0 <= pos) {
					address += getOpeSize(pos);
					while (p < n) {
						if (*items[p] == '=') {
							addExLabel(items[p]);
							break;
						}
						p++;
					}
				}
			}
		}
	}
	lastAddress = alignExLabels(address);
	fclose(fp);
	return NOERROR;
}


int pass2(char *fname)
{
	int		result = 0;
	word	address;
	FILE	*fp;
	char	objName[BUFSIZ];
	char	prnName[BUFSIZ];
	char	symName[BUFSIZ];

	makeObjName(fname, objName);
	makePrnName(fname, prnName);
	makeSymName(fname, symName);

	if ((fp = fopen(fname, "r")) == NULL) {
		return result;
	}
#ifdef __UNIX__
	if ((obj = fopen(objName, "w")) == NULL) {
#else
	if ((obj = fopen(objName, "wb")) == NULL) {
#endif
		fclose(fp);
		return result;
	}
	if ((prn = fopen(prnName, "w")) == NULL) {
		fclose(fp);
		fclose(obj);
		return result;
	}
	if ((sym = fopen(symName, "w")) == NULL) {
		fclose(fp);
		fclose(obj);
		fclose(prn);
		return result;
	}
	
	nError = 0;
	address = ORGADDR;
	for (lineNo = 1; fgets(inbuf, BUFSIZ, fp); lineNo++) {
		address = executeLineAssemble(address);
	}
	printExLabelsPrn(prn, address);
	outExLabelsObj();


	fprintf(prn, "\n\nLabels\n");
	printLabels(prn);
//	printExLabels(prn);

	printLabels(sym);
//	printExLabels(prn);

	fclose(fp);
	fclose(obj);
	fclose(prn);
	fclose(sym);
	return nError;
}

word getDCSize(char **items, int pos, int size)
{
	word	result;

	for (result = 0; pos < size; pos++) {
		if (*items[pos] == '\'') {
			result += strlen(items[pos]) - 2;
		} else if (*items[pos] != ',') {
			result++;
		}
	}
	return result;
}

void dispHelp()
{
}

