/*
**	execute.c
*/
#include <stdio.h>
#include <ctype.h>
#include "opencomet.h"

int execute(word address)
{
	int		result = 0;
	word	adr1 = getMem(address);
	word	adr2 = getMem(address + 1);
	word	ope  = adr1 & CMD_MASK;
	word	greg = (adr1 & GR_MASK) >> 4;
	word	xreg = (adr1 & XR_MASK);
	word	eadr, src, dst, val;
	uword	temp = 0;
	int		iv, is, id, i, n;

	src = val = 0;
	setPR(getPR() + 2);
	if (xreg & GR2_MASK) {
		eadr = xreg & ~GR2_MASK;
		dst = getReg(eadr);
	} else {
		if (7 < greg || 7 < xreg) {
			return 1;
		}
		eadr = adr2 + ((0 < xreg) ? getReg(xreg): 0);
		dst = getMem(eadr);
	}
	switch(ope) {
	case CMD_NOP:
		break;
	case CMD_LD:
		setReg(greg, dst);
		setFLAG(eadr);
		break;
	case CMD_ST:
		setMem(eadr, getReg(greg));
		break;
	case CMD_LAD:
		setReg(greg, eadr);
		break;
	case CMD_ADDA:
		src = getReg(greg);
		val =  src + dst;
		setReg(greg, val);
		setFLAG(val);
		is = (int) src;
		id = (int) dst;
		iv = src + dst;
		setOFLAG(iv < MINWORD || MAXWORD < iv);
		break;
	case CMD_ADDL:
		src = getReg(greg);
		val =  src + dst;
		setReg(greg, val);
		setFLAG(val);
		is = (int) src;
		id = (int) dst;
		iv = src + dst;
		setOFLAG(iv < 0 || MAXUWORD < iv);
		break;
	case CMD_SUBA:
		val =  getReg(greg) - dst;
		setReg(greg, val);
		setFLAG(val);
		is = (int) src;
		id = (int) dst;
		iv = is + id;
		setOFLAG(iv < MINWORD || MAXWORD < iv);
		break;
	case CMD_SUBL:
		src = getReg(greg);
		val =  src - dst;
		setReg(greg, val);
		setFLAG(val);
		is = (int) src;
		id = (int) dst;
		iv = is - id;
		setOFLAG(iv < 0 || MAXUWORD < iv);
		break;
	case CMD_AND:
		val =  getReg(greg) & dst;
		setReg(greg, val);
		setFLAG(val);
		break;
	case CMD_OR:
		val =  getReg(greg) | dst;
		setReg(greg, val);
		setFLAG(val);
		break;
	case CMD_XOR:
		val =  getReg(greg) ^ dst;
		setReg(greg, val);
		setFLAG(val);
		break;
	case CMD_CPA:
		val =  getReg(greg) - dst;
		setFLAG(val);
		is = (int) src;
		id = (int) dst;
		iv = is - id;
		setOFLAG(iv < MINWORD || MAXWORD < iv);		
		break;
	case CMD_CPL:
		val = (word) ((uword) getReg(greg) - (uword) dst);
		setFLAG(val);
		is = (int) src;
		id = (int) dst;
		iv = is - id;
		setOFLAG(iv < 0 || MAXUWORD < iv);
		break;
	case CMD_SLA:
		val = getReg(greg);
		iv = val & 0x8000;
		val <<= eadr;
		setReg(greg, (word) val);
		setFLAG(val);
		setOFLAG(iv);
		break;		
	case CMD_SRA:
		val = getReg(greg);
		iv = val & 0x0001;
		val >>= eadr;
		setReg(greg, (word) val);
		setFLAG(val);
		setOFLAG(iv);
		break;		
	case CMD_SLL:
		temp = getReg(greg);
		iv = temp & 0x8000;
		temp <<= eadr;
		setReg(greg, (word) temp);
		setFLAG(temp);
		setOFLAG(iv);
		break;
	case CMD_SRL:
		temp = getReg(greg);
		iv = temp & 0x0001;
		temp >>= eadr;
		setReg(greg, (word) temp);
		setFLAG(temp);
		setOFLAG(iv);
		break;
	case CMD_JPL:
		if (!(getFlag() & (ZFLG | SFLG))) {
			setPR(eadr);
		}
		break;
	case CMD_JMI:
		if (getFlag() & SFLG) {
			setPR(eadr);
		}
		break;
	case CMD_JNZ:
		if (!(getFlag() & ZFLG)) {
			setPR(eadr);
		}
		break;
	case CMD_JZE:
		if (getFlag() & ZFLG) {
			setPR(eadr);
		}
		break;
	case CMD_JOV:
		if (getFlag() & OFLG) {
			setPR(eadr);
		}
		break;
	case CMD_JUMP:
		setPR(eadr);
		break;
	case CMD_PUSH:
		pushData(eadr);
		break;
	case CMD_POP:
		setReg(greg, popData());
		break;
	case CMD_CALL:
		pushData(getPR());
		setPR(eadr);
		break;
	case CMD_RET:
		setPR(popData());
		break;
	case CMD_INPUT:
	case CMD_OUTPUT:
	case CMD_RPUSH:
		pushData(getReg(1));
		pushData(getReg(2));
		pushData(getReg(3));
		pushData(getReg(4));
		pushData(getReg(5));
		pushData(getReg(6));
		pushData(getReg(7));
		break;
	case CMD_RPOP:
		setReg(7, popData());
		setReg(6, popData());
		setReg(5, popData());
		setReg(4, popData());
		setReg(3, popData());
		setReg(2, popData());
		setReg(1, popData());
		break;
	case CMD_SVC:
	case CMD_IN:
	case CMD_OUT:
		n = getMem(adr1 & 0x00ff);
		if (0 < n) {
			printf("OUTPUT> \"");
		}
		for (i = 0; i < n; i++) {
			val = getMem(adr2 + i);
			if (isprint(val)) {
				printf("%c", val);
			} else {
				printf(".");
			}
		}
		printf("\"\n");
		break;
	default:
		break;
	}

	return result;
}

void setFLAG(word value)
{
	word	flag = getFlag();
	
	if (value < 0) {
		flag |= SFLG;
	} else {
		flag &= ~SFLG;
	}
	if (value == 0) {
		flag |= ZFLG;
	} else {
		flag &= ~ZFLG;
	}
	flag &= ~OFLG;
	
	setFlag(flag);
}

void setOFLAG(int value)
{
	word	flag = getFlag();
	
	if (value) {
		flag |= OFLG;
	} else {
		flag &= ~OFLG;
	}
	setFlag(flag);
}

